package com.demo.index;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import com.jfinal.core.Controller;
import com.jfinal.plugin.activerecord.Db;

/**
 * 秒杀/MySQL 乐观锁实现方式
 * 
 * @author sun
 */
public class SecKillForOLController extends Controller {
	
	public static int goodsTotalNum = 100;

	public void index() {
		//每一次新的请求都会模拟抢购100件商品，所以先恢复商品的数量,和版本号，这个可以不要，也可以手动去更新数据库
		Db.update("update t_goods set num = ?, version = 1 where id = 1", goodsTotalNum);
		
		SecKillOL sk = new SecKillOL();
		// 模拟120个人抢购。 模拟每60人分别在不同的服务器进行发起请求，这样我们的synchronized锁才能体现出来
		//如果我们只new一个SecKillOl，那么相当于synchronized锁了。MySQL乐观锁无法体现出来。
		for (int i = 1; i <= 60; i++) {
			new Thread(sk, i + " 号人").start();
		}
		SecKillOL sk2 = new SecKillOL();
		for (int i = 61; i <= 120; i++) {
			new Thread(sk2, i + " 号人").start();
		}
		renderHtml("<center><h2>抢购成功!</h2></center>");
	}
}

/**
 * 这里我们只是演示数据库悲观锁的实现方式，至于事务等其他的都不添加了。
 * 乐观锁本来就不太适合秒杀等业务处理，因为乐观的本质就是： 我很乐观，我先查询，我不认为别人会和我抢。
 * @author sun
 */
class SecKillOL implements Runnable {

	@Override
	public void run() {
		synchronized (this) {
			//如果version字段对不上，回滚进行再一次提交，我这里就设置轮询十次，如果还是没有对上，则相当于抢购失败
			//这块的乐观锁异常处理根据实际情况来做，比如你也可以增加while(true)直到成功为止。
			for(int i=0;i<10;i++){
				Connection conn = null;
				PreparedStatement ps = null;
				try {
					conn = Db.use().getConfig().getConnection();
					ps = conn.prepareStatement("select * from t_goods where id = 1");
					ResultSet rs = ps.executeQuery();
					Integer num = 0;
					Integer version = null;
					while (rs.next()) {
						num = rs.getInt("num");
						version = rs.getInt("version");
					}
					if (num > 0) {
						PreparedStatement ps2 = conn.prepareStatement("update t_goods set num=num-1, version=version+1 where id = 1 and version = ?");
						ps2.setInt(1, version);
						int result = ps2.executeUpdate();
						if(result>0){
							System.out.println(Thread.currentThread().getName() + " 抢到了第" + (SecKillForOLController.goodsTotalNum - num + 1) + "商品");
							break;
						}else{
							System.err.println(Thread.currentThread().getName() + " 有人抢到了我前边，导致我无法进行更新，我再尝试一遍");
						}
					}else{
						System.err.println(Thread.currentThread().getName() + " 太悲剧了，没抢到商品...");
					}
				} catch (SQLException e1) {
					e1.printStackTrace();
				} finally {
					try {
						ps.close();
					} catch (SQLException e) {}
					try {
						conn.close();
					} catch (SQLException e) {}
				}
			}
		}
	}
}