package com.demo.index;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import com.jfinal.core.Controller;
import com.jfinal.plugin.activerecord.Db;

/**
 * 秒杀/MySQL 悲观锁实现方式
 * 
 * @author sun
 */
public class SecKillForPLController extends Controller {
	
	public static int goodsTotalNum = 100;

	public void index() {
		//每一次新的请求都会模拟抢购100件商品，所以先恢复商品的数量,这个可以不要，也可以手动去更新数据库
		Db.update("update t_goods set num = ? where id = 1", goodsTotalNum);
		
		SecKillPL sk = new SecKillPL();
		// 模拟120个人抢购
		for (int i = 1; i <= 120; i++) {
			new Thread(sk, i + " 号人").start();
		}
		renderHtml("<center><h2>抢购成功!</h2></center>");
	}
}

/**
 * 这里我们只是演示数据库悲观锁的实现方式，至于事务等其他的都不添加了。
 * 
 * @author sun
 */
class SecKillPL implements Runnable {

	@Override
	public void run() {
		Connection conn = null;
		PreparedStatement ps = null;
		try {
			conn = Db.use().getConfig().getConnection();
			conn.setAutoCommit(false);
			ps = conn.prepareStatement("select * from t_goods where id = 1 for update");
			ResultSet rs = ps.executeQuery();
			Integer num = 0;
			while (rs.next()) {
				num = rs.getInt("num");
			}
			if (num > 0) {
				PreparedStatement ps2 = conn.prepareStatement("update t_goods set num=num-1 where id = 1");
				ps2.execute();
				System.out.println(Thread.currentThread().getName() + " 抢到了第" + (SecKillForPLController.goodsTotalNum - num + 1) + "商品");
				conn.commit();
			}else{
				System.err.println(Thread.currentThread().getName() + " 太悲剧了，没抢到商品...");
			}
		} catch (SQLException e1) {
			e1.printStackTrace();
		} finally {
			try {
				ps.close();
			} catch (SQLException e) {}
			try {
				conn.close();
			} catch (SQLException e) {}
		}
	}
}